@@ -1,10 +1,10 @@
 /*
      This file is part of GNUnet.
-     (C) 2010, 2012 Christian Grothoff (and other contributing authors)
+     (C) 2010-2013 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 2, or (at your
+     by the Free Software Foundation; either version 3, or (at your
      option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
@@ -32,6 +32,8 @@
 #include <unique/unique.h>
 #endif
 
+#include <gnunet/gnunet_arm_service.h>
+
 /**
  * How many block requests can we have outstanding in parallel at a time by default?
  */
@@ -42,6 +44,7 @@
  */
 #define DEFAULT_MAX_PARALLEL_DOWNLOADS 128
 
+#define SERVICE_LIST_TIMEOUT GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, 1)
 
 /**
  * Should gnunet-fs-gtk start in tray mode?
@@ -58,6 +61,16 @@ static struct GNUNET_GTK_MainLoop *ml;
  */
 static struct GNUNET_FS_Handle *fs;
 
+/**
+ * Handle for ARM monitoring.
+ */
+static struct GNUNET_ARM_MonitorHandle *armon;
+
+/**
+ * Handle for ARM controlling.
+ */
+static struct GNUNET_ARM_Handle *arm;
+
 /**
  * Context for main window.
  */
@@ -68,6 +81,7 @@ static struct GNUNET_GTK_MainWindowContext main_context;
 static UniqueApp *unique_app;
 #endif
 
+
 struct GNUNET_GTK_MainWindowContext *
 GNUNET_FS_GTK_get_main_context ()
 {
@@ -87,6 +101,19 @@ GNUNET_FS_GTK_get_fs_handle ()
 }
 
 
+/**
+ * Remember FS handle if we don't have one yet.
+ *
+ * @param fsh file sharing handle to use
+ */
+void
+GNUNET_FS_GTK_set_fs_handle (struct GNUNET_FS_Handle *fsh)
+{
+  if (NULL == fs)
+    fs = fsh;
+}
+
+
 /**
  * Get our configuration.
  *
@@ -156,21 +183,21 @@ main_window_save_position (GtkWidget *main_window)
   gtk_window_get_position (GTK_WINDOW (main_window), &window_x, &window_y);
   gtk_window_get_size (GTK_WINDOW (main_window), &window_width, &window_height);
 
-  GNUNET_CONFIGURATION_set_value_number (cfg, "gnunet-gtk",
+  GNUNET_CONFIGURATION_set_value_number (cfg, "gnunet-fs-gtk",
 					 "MAIN_WINDOW_X", window_x);
-  GNUNET_CONFIGURATION_set_value_number (cfg, "gnunet-gtk",
+  GNUNET_CONFIGURATION_set_value_number (cfg, "gnunet-fs-gtk",
 					 "MAIN_WINDOW_Y", window_y);
-  GNUNET_CONFIGURATION_set_value_number (cfg, "gnunet-gtk",
+  GNUNET_CONFIGURATION_set_value_number (cfg, "gnunet-fs-gtk",
 					 "MAIN_WINDOW_WIDTH", window_width);
-  GNUNET_CONFIGURATION_set_value_number (cfg, "gnunet-gtk",
+  GNUNET_CONFIGURATION_set_value_number (cfg, "gnunet-fs-gtk",
 					 "MAIN_WINDOW_HEIGHT", window_height);
-  GNUNET_CONFIGURATION_set_value_string (cfg, "gnunet-gtk",
-					 "MAIN_WINDOW_MAXIMIZED", 
+  GNUNET_CONFIGURATION_set_value_string (cfg, "gnunet-fs-gtk",
+					 "MAIN_WINDOW_MAXIMIZED",
 					 (maximized == GNUNET_YES) ? "YES" : "NO");
 
   cfgDefault = GNUNET_CONFIGURATION_create ();
   (void) GNUNET_CONFIGURATION_load (cfgDefault, NULL); /* load defaults only */
-  GNUNET_CONFIGURATION_write_diffs (cfgDefault, cfg, 
+  GNUNET_CONFIGURATION_write_diffs (cfgDefault, cfg,
 				    GNUNET_GTK_main_loop_get_configuration_file (ml));
   GNUNET_CONFIGURATION_destroy (cfgDefault);
 }
@@ -181,6 +208,8 @@ main_window_save_position (GtkWidget *main_window)
  * and saves these into user's config file.
  *
  * @param main_window main window widget
+ * @param event the event that we are handling
+ * @param user_data main window context
  */
 gboolean
 GNUNET_GTK_main_window_configure_event_cb (GtkWidget *main_window,
@@ -196,24 +225,77 @@ GNUNET_GTK_main_window_configure_event_cb (GtkWidget *main_window,
 
 /**
  * Task run on shutdown.
- * FIXME-STYLE: does this need to be a separate task!?
  *
  * @param cls NULL
  * @param tc scheduler context, unused
  */
 static void
-shutdown_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
-{  
+shutdown_task (void *cls,
+               const struct GNUNET_SCHEDULER_TaskContext *tc)
+{
+  struct SearchLookup *sl;
+  struct PseuLookupContext *lctx;
+  struct SearchResult *sr;
+
   GNUNET_GTK_tray_icon_destroy ();
-  if (fs != NULL)
+  while (NULL != (sr = pl_head))
+  {
+    GNUNET_FS_probe_stop (sr->probe);
+    sr->probe = NULL;
+    GNUNET_CONTAINER_DLL_remove (pl_head,
+                                 pl_tail,
+                                 sr);
+  }
+  if (NULL != fs)
   {
     GNUNET_FS_stop (fs);
     fs = NULL;
   }
+  if (NULL != armon)
+  {
+    GNUNET_ARM_monitor_disconnect_and_free (armon);
+    armon = NULL;
+  }
+  if (NULL != arm)
+  {
+    GNUNET_ARM_disconnect_and_free (arm);
+    arm = NULL;
+  }
   GNUNET_FS_GTK_close_uri_tab_ ();
   if (NULL != ml)
+  {
     GNUNET_GTK_main_loop_quit (ml);
-  ml = NULL;
+    ml = NULL;
+  }
+  if (NULL != main_context.id_op)
+  {
+    GNUNET_IDENTITY_cancel (main_context.id_op);
+    main_context.id_op = NULL;
+  }
+  if (NULL != main_context.identity)
+  {
+    GNUNET_IDENTITY_disconnect (main_context.identity);
+    main_context.identity = NULL;
+  }
+  while (NULL != (sl = main_context.sl_head))
+    abort_search_lookup (sl);
+  while (NULL != (lctx = main_context.lctx_head))
+    abort_pseu_lookup (lctx);
+  if (NULL != main_context.zm)
+  {
+    GNUNET_NAMESTORE_zone_monitor_stop (main_context.zm);
+    main_context.zm = NULL;
+  }
+  if (NULL != main_context.gns)
+  {
+    GNUNET_GNS_disconnect (main_context.gns);
+    main_context.gns = NULL;
+  }
+  if (NULL != main_context.sks_zone)
+  {
+    GNUNET_free (main_context.sks_zone);
+    main_context.sks_zone = NULL;
+  }
 }
 
 
@@ -235,6 +317,7 @@ GNUNET_FS_GTK_menu_quit_activate_cb (GtkMenuItem *object,
  * Callback invoked if the application is supposed to exit.
  *
  * @param object origin of the quit event, unused
+ * @param event the delete event
  * @param user_data global builder instance, unused
  */
 void
@@ -299,15 +382,15 @@ process_paste (GtkClipboard *cb,
  *         TRUE if there is now a pop-up menu
  */
 gboolean
-GNUNET_FS_GTK_main_window_button_press_event (GtkWidget * widget, 
+GNUNET_FS_GTK_main_window_button_press_event (GtkWidget * widget,
 					      GdkEvent * event,
 					      gpointer user_data)
 {
   GdkEventButton *event_button = (GdkEventButton *) event;
   GtkClipboard *cb;
 
-  if ( (event->type != GDK_BUTTON_PRESS) ||
-       (event_button->button != 2) )
+  if ( (GDK_BUTTON_PRESS != event->type) ||
+       (2 != event_button->button) )
     return FALSE;
   cb = gtk_clipboard_get (gdk_atom_intern ("PRIMARY", FALSE));
   gtk_clipboard_request_text (cb, &process_paste, NULL);
@@ -373,11 +456,220 @@ unique_app_message_cb (UniqueApp *app,
 #endif
 
 
+static char *
+format_service_list (unsigned int count, const char *const*list)
+{
+  size_t len = 0;
+  int i;
+  const char *header = _("GNUnet node appears to be on.");
+  const char *listheader = _("Currently running services:\n");
+  char *result;
+  char *p;
+  int r;
+
+  len = strlen (header) + 1;
+  if (list)
+  {
+    len += strlen (listheader);
+    for (i = 0; i < count; i++)
+      len += strlen (list[i]) + 1;
+  }
+  len += 1;
+  result = p = GNUNET_malloc (len * sizeof (char));
+  r = GNUNET_snprintf (p, (size_t) (result + len - p), "%s\n", header);
+  if (r < 0)
+  {
+    GNUNET_free (result);
+    return NULL;
+  }
+  p += r;
+  if (list)
+  {
+    r = GNUNET_snprintf (p, (size_t) (result + len - p), "%s", listheader);
+    if (r < 0)
+    {
+      GNUNET_free (result);
+      return NULL;
+    }
+    p += r;
+    for (i = 0; i < count; i++)
+    {
+      size_t l = strlen (list[i]);
+      memcpy (p, list[i], l);
+      p += l;
+      if (i + 1 < count)
+      {
+        p[0] = '\n';
+        p += 1;
+      }
+    }
+  }
+  p[0] = '\0';
+  return result;
+}
+
+
+static void
+service_list_callback (void *cls,
+		       enum GNUNET_ARM_RequestStatus rs,
+		       unsigned int count,
+		       const char *const*list)
+{
+  char *service_list;
+
+  if ((GNUNET_ARM_REQUEST_SENT_OK != rs) || (NULL == list))
+    return;
+  service_list = format_service_list (count, list);
+  GNUNET_FS_GTK_update_connection_indicator (cls, TRUE, service_list);
+  GNUNET_free_non_null (service_list);
+}
+
+
+static void
+arm_connection_state_change (void *cls,
+			     int connected)
+{
+  char *service_list;
+
+  if (connected)
+  {
+    service_list = format_service_list (0, NULL);
+    GNUNET_FS_GTK_update_connection_indicator (cls, TRUE, service_list);
+    GNUNET_free_non_null (service_list);
+    GNUNET_ARM_request_service_list (arm, SERVICE_LIST_TIMEOUT,
+				     &service_list_callback, cls);
+  }
+  else
+    GNUNET_FS_GTK_update_connection_indicator (cls, FALSE,
+        _("Can't connect to the Automatic Restart Manager service."));
+}
+
+
+static void
+service_status_change (void *cls,
+		       const char *service,
+		       enum GNUNET_ARM_ServiceStatus status)
+{
+  /* Very crude, we can probably do better.
+   * Maybe keep a list of running services, and modify it in response
+   * to service status changes, then update the indicator,
+   * without requesting a list from ARM every goddamned time?
+   */
+  GNUNET_ARM_request_service_list (arm,
+				   SERVICE_LIST_TIMEOUT,
+				   &service_list_callback, cls);
+}
+
+
+/**
+ * Process a record that was stored in the namestore in the
+ * "sks_zone".  Adds (or removes) the respective label to the
+ * drop-down menu for the SKS search by manipulating the
+ * "namespace_label_liststore".
+ *
+ * @param cls closure
+ * @param zone private key of the zone; NULL on disconnect
+ * @param label label of the records; NULL on disconnect
+ * @param rd_count number of entries in @a rd array; 0 on removal
+ * @param rd array of records for the label
+ */
+static void
+monitor_zone_records (void *cls,
+		      const struct GNUNET_CRYPTO_EcdsaPrivateKey *zone,
+		      const char *label,
+		      unsigned int rd_count,
+		      const struct GNUNET_GNSRECORD_Data *rd)
+{
+  GtkListStore *ls;
+  GtkTreeModel *tm;
+  GtkTreeIter iter;
+  gchar *id;
+  gchar *label_gnu;
+
+  ls = GTK_LIST_STORE (GNUNET_FS_GTK_get_main_window_object ("namespace_label_liststore"));
+  if (NULL == zone)
+  {
+    gtk_list_store_clear (ls);
+    return;
+  }
+  label_gnu = g_strdup_printf ("%s.%s",
+			       label,
+			       "gnu");
+  if (0 == rd_count)
+  {
+    tm = GTK_TREE_MODEL (ls);
+    if (gtk_tree_model_get_iter_first (tm, &iter))
+      do
+	{
+	  gtk_tree_model_get (tm, &iter,
+			      0, &id,
+			      -1);
+	  if (0 == strcmp (id, label_gnu))
+	    {
+	      GNUNET_assert (gtk_list_store_remove (ls, &iter));
+	      g_free (id);
+	      g_free (label_gnu);
+	      return;
+	    }
+	  g_free (id);
+	}
+      while (gtk_tree_model_iter_next (tm, &iter));
+    GNUNET_break (0);
+    return;
+  }
+  gtk_list_store_insert_with_values (ls,
+				     &iter,
+				     -1,
+				     0, label_gnu,
+				     -1);
+  g_free (label_gnu);
+}
+
+
+/**
+ * The identity service has given us the ego we should use for resolving
+ * namepace names.
+ *
+ * @param cls closure
+ * @param ego ego handle, NULL for none
+ * @param ctx context for application to store data for this ego
+ *                 (during the lifetime of this process, initially NULL)
+ * @param name name assigned by the user for this ego,
+ *                   NULL if the user just deleted the ego and it
+ *                   must thus no longer be used
+ */
+static void
+handle_sks_zone_identity (void *cls,
+			  struct GNUNET_IDENTITY_Ego *ego,
+			  void **ctx,
+			  const char *name)
+{
+  main_context.id_op = NULL;
+
+  if (NULL == ego)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+		_("No default ego specified for `fs-sks` service, will not enable namespace search.\n"));
+    return;
+  }
+  main_context.sks_zone = GNUNET_new (struct GNUNET_CRYPTO_EcdsaPrivateKey);
+  *main_context.sks_zone = *GNUNET_IDENTITY_ego_get_private_key (ego);
+  gtk_widget_show (GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object ("main_window_search_namespace_label")));
+  gtk_widget_show (GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object ("main_window_search_namespace_combobox")));
+  main_context.zm = GNUNET_NAMESTORE_zone_monitor_start (main_context.cfg,
+							 main_context.sks_zone,
+                                                         GNUNET_YES,
+							 &monitor_zone_records,
+							 NULL,
+							 NULL);
+}
+
+
 /**
  * Actual main function run right after GNUnet's scheduler
  * is initialized.  Initializes up GTK and Glade.
  *
- * @param cls handle to the main loop ('struct GNUNET_GTK_MainLoop')
+ * @param cls handle to the main loop (`struct GNUNET_GTK_MainLoop`)
  * @param tc scheduler context, unused
  */
 static void
@@ -393,12 +685,14 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 
   ml = cls;
   /* setup main context */
-  if (GNUNET_OK != GNUNET_GTK_main_loop_build_window (cls, &main_context))
+  if (GNUNET_OK != GNUNET_GTK_main_loop_build_window (ml, &main_context))
     return;
-  main_context.builder = GNUNET_GTK_main_loop_get_builder (cls);
-  main_context.cfg = GNUNET_GTK_main_loop_get_configuration (cls);
+  main_context.builder = GNUNET_GTK_main_loop_get_builder (ml);
+  main_context.cfg = GNUNET_GTK_main_loop_get_configuration (ml);
   main_context.search_ns_treestore = GTK_TREE_STORE (GNUNET_FS_GTK_get_main_window_object ("main_window_search_namespace_treestore"));
   main_context.main_window = GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window"));
+  main_context.main_window = GNUNET_GTK_plug_me ("GNUNET_FS_GTK_PLUG",
+                                                 main_context.main_window);
   main_context.ns_selector_treeview = GTK_TREE_VIEW (GNUNET_FS_GTK_get_main_window_object ("namespace_selector_treeview"));
   main_context.ns_selector_window = GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object ("namespace_selector_window"));
   main_context.ns_dropdown_button = GTK_TOGGLE_BUTTON (GNUNET_FS_GTK_get_main_window_object ("main_window_search_namespace_dropdown_button"));
@@ -412,7 +706,7 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   main_context.preview_image = GTK_IMAGE (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window_preview_image"));
   main_context.md_liststore = GTK_LIST_STORE (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_meta_data_list_store"));
   main_context.md_treeview = GTK_TREE_VIEW (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window_metadata_treeview"));
-  main_context.ns_callback_registered = GNUNET_NO;
+  main_context.ns_discovery_handle = NULL;
 
   main_context.download_location_chooser = GTK_FILE_CHOOSER (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_search_frame_download_location_chooser"));
   main_context.download_name_entry = GTK_ENTRY (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_search_frame_download_filename_entry"));
@@ -423,6 +717,8 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 
   main_context.notebook = GTK_NOTEBOOK (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window_notebook"));
 
+  main_context.connection_indicator = GTK_IMAGE (GNUNET_FS_GTK_get_main_window_object ("GNUNET_FS_GTK_main_window_connection_indicator"));
+
   GNUNET_GTK_set_icon_search_path ();
   GNUNET_GTK_setup_nls ();
 
@@ -435,17 +731,18 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 
   /* setup main window */
   maximized = GNUNET_CONFIGURATION_get_value_yesno (main_context.cfg,
-						    "gnunet-gtk", "MAIN_WINDOW_MAXIMIZED");
+						    "gnunet-fs-gtk",
+                                                    "MAIN_WINDOW_MAXIMIZED");
   if (GNUNET_SYSERR == maximized)
     maximized = GNUNET_YES;
   if ( (GNUNET_NO == maximized) &&
-       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-gtk",
+       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-fs-gtk",
 							    "MAIN_WINDOW_X", &window_x)) &&
-       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-gtk",
+       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-fs-gtk",
 							    "MAIN_WINDOW_Y", &window_y)) &&
-       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-gtk",
+       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-fs-gtk",
 							    "MAIN_WINDOW_WIDTH", &window_width)) &&
-       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-gtk",
+       (GNUNET_OK == GNUNET_CONFIGURATION_get_value_number (main_context.cfg, "gnunet-fs-gtk",
 							    "MAIN_WINDOW_HEIGHT", &window_height)) )
   {
     gtk_window_move (GTK_WINDOW (main_context.main_window), window_x, window_y);
@@ -456,16 +753,16 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     /* If anything is wrong - play safe and show it maximized */
     gtk_window_maximize (GTK_WINDOW (main_context.main_window));
   }
-  
+
   /* Allow multiple selection in metadata view; */
   /* FIXME-GTK3: this can be done within (modern versions of) glade */
   gtk_tree_selection_set_mode (gtk_tree_view_get_selection (main_context.md_treeview),
                                GTK_SELECTION_MULTIPLE);
-
-  GNUNET_GTK_tray_icon_create (ml,
-			       GTK_WINDOW (main_context.main_window),
-                               "gnunet-fs-gtk",
-                               "gnunet-fs-gtk");
+  if (NULL == getenv ("GNUNET_FS_GTK_PLUG"))
+    GNUNET_GTK_tray_icon_create (ml,
+                                 GTK_WINDOW (main_context.main_window),
+                                 "gnunet-fs-gtk",
+                                 "gnunet-fs-gtk");
 
   /* FIXME: should these '1's be here? Maybe better to put them into
    * default config files?
@@ -479,7 +776,7 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 
   /* initialize file-sharing */
   fs = GNUNET_FS_start (main_context.cfg, "gnunet-fs-gtk",
-			&GNUNET_GTK_fs_event_handler, NULL, 
+			&GNUNET_GTK_fs_event_handler, NULL,
 			GNUNET_FS_FLAGS_PERSISTENCE | GNUNET_FS_FLAGS_DO_PROBES,
                         GNUNET_FS_OPTIONS_DOWNLOAD_PARALLELISM,
                         (unsigned int) dl_parallel,
@@ -491,8 +788,14 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     GNUNET_GTK_main_loop_quit (cls);
     return;
   }
-
-  GNUNET_GTK_main_window_refresh_ns_list (&main_context);
+  arm = GNUNET_ARM_connect (main_context.cfg, &arm_connection_state_change, &main_context);
+  armon = GNUNET_ARM_monitor (main_context.cfg, service_status_change, &main_context);
+  main_context.gns = GNUNET_GNS_connect (main_context.cfg);
+  main_context.identity = GNUNET_IDENTITY_connect (main_context.cfg, NULL, NULL);
+  main_context.id_op = GNUNET_IDENTITY_get (main_context.identity,
+					    "fs-sks",
+					    &handle_sks_zone_identity,
+					    NULL);
 #if HAVE_LIBUNIQUE
   unique_app_watch_window (unique_app, GTK_WINDOW (main_context.main_window));
   g_signal_connect (unique_app, "message-received",
@@ -504,6 +807,26 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     gtk_widget_show (main_context.main_window);
     gtk_window_present (GTK_WINDOW (main_context.main_window));
   }
+
+  {
+    char *const *argv;
+    int argc;
+    int i;
+
+    GNUNET_GTK_main_loop_get_args (ml, &argc, &argv);
+
+    for (i = 0; i < argc; i++)
+      {
+	if (GNUNET_OK !=
+	    GNUNET_FS_GTK_handle_uri_string (argv[i],
+					     1 /* anonymity level */))
+	  fprintf (stderr,
+		   "Invalid URI `%s'\n",
+		   argv[i]);
+      }
+  }
+
+
   GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
 				&shutdown_task, NULL);
 }
@@ -517,7 +840,7 @@ main (int argc, char **argv)
      gettext_noop ("start in tray mode"), 0,
      &GNUNET_GETOPT_set_one, &tray_only},
     GNUNET_GETOPT_OPTION_END
-  };  
+  };
 #if HAVE_LIBUNIQUE
   int arge;
 
@@ -536,7 +859,7 @@ main (int argc, char **argv)
       UniqueMessageData *msg;
 
       msg = unique_message_data_new ();
-      unique_message_data_set_text (msg, argv[arge], strlen (argv[arge])+1);      
+      unique_message_data_set_text (msg, argv[arge], strlen (argv[arge])+1);
       if (UNIQUE_RESPONSE_OK == response)
 	response = unique_app_send_message (unique_app,
 					    UNIQUE_OPEN,
@@ -548,11 +871,15 @@ main (int argc, char **argv)
 
     return (UNIQUE_RESPONSE_OK == response) ? 0 : 1;
   }
+
 #endif
   if (GNUNET_OK !=
-      GNUNET_GTK_main_loop_start ("gnunet-fs-gtk", "GTK GUI for GNUnet", argc,
-                                  argv, options,
-                                  "gnunet_fs_gtk_main_window.glade", &run))
+      GNUNET_GTK_main_loop_start ("gnunet-fs-gtk",
+				  "GTK GUI for GNUnet",
+				  argc, argv,
+				  options,
+                                  "gnunet_fs_gtk_main_window.glade",
+				  &run))
   {
 #if HAVE_LIBUNIQUE
     g_object_unref (unique_app);
