@@ -4,7 +4,7 @@
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 2, or (at your
+     by the Free Software Foundation; either version 3, or (at your
      option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
@@ -20,7 +20,7 @@
 
 /**
  * @file src/fs/gnunet-fs-gtk_download-save-as.c
- * @brief functions for managing the 'save as' dialog 
+ * @brief functions for managing the 'save as' dialog
  * @author Christian Grothoff
  */
 #include "gnunet-fs-gtk_download-save-as.h"
@@ -43,7 +43,7 @@ struct DownloadAsDialogContext
    * Builder for the dialog.
    */
   GtkBuilder *builder;
-  
+
   /**
    * Main dialog object.
    */
@@ -97,13 +97,13 @@ GNUNET_FS_GTK_free_download_entry (struct DownloadEntry *de)
   GNUNET_assert (NULL == de->dc);
   if (NULL != de->sr)
   {
-    GNUNET_assert (de->sr->download == de); 
+    GNUNET_assert (de->sr->download == de);
     de->sr->download = NULL;
     de->sr = NULL;
   }
   GNUNET_free_non_null (de->filename);
   GNUNET_FS_uri_destroy (de->uri);
-  GNUNET_free (de);  
+  GNUNET_free (de);
 }
 
 
@@ -136,7 +136,7 @@ GNUNET_GTK_save_as_dialog_delete_event_cb (GtkWidget * widget, GdkEvent * event,
  * @param user_data the 'structDownloadAsDialogContext' of the dialog
  */
 void
-GNUNET_GTK_save_as_dialog_response_cb (GtkDialog * dialog, 
+GNUNET_GTK_save_as_dialog_response_cb (GtkDialog * dialog,
 				       gint response_id,
                                        gpointer user_data)
 {
@@ -148,7 +148,7 @@ GNUNET_GTK_save_as_dialog_response_cb (GtkDialog * dialog,
   if (GTK_RESPONSE_OK != response_id)
   {
     GNUNET_FS_GTK_free_download_entry (de);
-    gtk_widget_destroy (GTK_WIDGET (dialog));    
+    gtk_widget_destroy (GTK_WIDGET (dialog));
     clean_up_download_as_context (dlc);
     return;
   }
@@ -198,7 +198,7 @@ open_saveas_dialog (struct DownloadEntry *de, int download_directly)
   GtkWidget *cb;
   GtkWidget *hb;
 
-  dlc = GNUNET_malloc (sizeof (struct DownloadAsDialogContext));
+  dlc = GNUNET_new (struct DownloadAsDialogContext);
   dlc->de = de;
   dlc->download_directly = download_directly;
   dlc->builder =
@@ -316,28 +316,28 @@ GNUNET_FS_GTK_download_context_start_download (struct DownloadEntry *de)
   if (de->is_recursive)
     opt |= GNUNET_FS_DOWNLOAD_OPTION_RECURSIVE;
   len = GNUNET_FS_uri_chk_get_file_size (de->uri);
-  if (NULL != de->sr->result)
+  if ((NULL != de->sr) && (NULL != de->sr->result))
   {
     GNUNET_break (NULL !=
-                  GNUNET_FS_download_start_from_search (fs, 
+                  GNUNET_FS_download_start_from_search (fs,
 							de->sr->result,
                                                         de->filename,
                                                         NULL /* tempname */ ,
                                                         0 /* offset */ ,
-                                                        len, 
+                                                        len,
 							de->anonymity, opt,
                                                         de));
   }
   else
   {
     GNUNET_break (NULL !=
-                  GNUNET_FS_download_start (fs, 
-					    de->uri, 
+                  GNUNET_FS_download_start (fs,
+					    de->uri,
 					    NULL /* meta data */,
                                             de->filename, NULL /* tempname */ ,
                                             0 /* offset */ ,
-                                            len, 
-					    de->anonymity, opt, 
+                                            len,
+					    de->anonymity, opt,
 					    de,
                                             (NULL != de->pde) ? de->pde->dc : NULL));
   }
