@@ -4,7 +4,7 @@
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 2, or (at your
+     by the Free Software Foundation; either version 3, or (at your
      option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
@@ -27,7 +27,7 @@
 #define GNUNET_FS_GTK_EVENT_HANDLER_H
 
 #include "gnunet-fs-gtk_common.h"
-
+#include "gnunet-fs-gtk.h"
 
 /**
  * State we keep for each (search) result entry in the
@@ -68,7 +68,7 @@ struct SearchTab
 
   /**
    * GtkBuilder object for the search tab.
-   */ 
+   */
   GtkBuilder *builder;
 
   /**
@@ -187,6 +187,19 @@ struct DownloadEntry
  */
 struct SearchResult
 {
+
+  /**
+   * If this search result has a manually-activated probe,
+   * we keep it in the 'pr_head' list.
+   */
+  struct SearchResult *next;
+
+  /**
+   * If this search result has a manually-activated probe,
+   * we keep it in the 'pr_head' list.
+   */
+  struct SearchResult *prev;
+
   /**
    * Where in the tab is this result?
    */
@@ -203,6 +216,13 @@ struct SearchResult
    */
   struct GNUNET_FS_SearchResult *result;
 
+  /**
+   * Associated search result we generated for probing;
+   * thus, we need to run "GNUNET_FS_probe_cancel" on it
+   * once we are done.  Only used if 'result' is NULL.
+   */
+  struct GNUNET_FS_SearchResult *probe;
+
   /**
    * Associated download, or NULL for none.
    */
@@ -210,16 +230,30 @@ struct SearchResult
 };
 
 
+/**
+ * Head of search results with a manually-activated probe.
+ */
+extern struct SearchResult *pl_head;
+
+/**
+ * Tail of search results with a manually-activated probe.
+ */
+extern struct SearchResult *pl_tail;
+
+
+
 /**
  * Setup a new top-level entry in the URI/orphan tab.  If necessary, create
  * the URI tab first.
  *
+ * @param anonymity anonymity level to use for probes
  * @param meta metadata for the new entry
  * @param uri URI for the new entry
  * @return the search result that was set up
  */
 struct SearchResult *
-GNUNET_GTK_add_to_uri_tab (const struct GNUNET_CONTAINER_MetaData *meta,
+GNUNET_GTK_add_to_uri_tab (uint32_t anonymity,
+			   const struct GNUNET_CONTAINER_MetaData *meta,
                            const struct GNUNET_FS_Uri *uri);
 
 
@@ -227,8 +261,9 @@ GNUNET_GTK_add_to_uri_tab (const struct GNUNET_CONTAINER_MetaData *meta,
  * Add a search result to the given search tab.
  *
  * @param tab search tab to extend, never NULL
+ * @param anonymity anonymity level to use for probes for this result
  * @param parent_rr reference to parent entry in search tab, NULL for normal
- *                  search results, 
+ *                  search results,
  * @param uri uri to add, can be NULL for top-level entry of a directory opened from disk
  *                        (in this case, we don't know the URI and should probably not
  *                         bother to calculate it)
@@ -240,7 +275,8 @@ GNUNET_GTK_add_to_uri_tab (const struct GNUNET_CONTAINER_MetaData *meta,
  *                model at 'iter')
  */
 struct SearchResult *
-GNUNET_GTK_add_search_result (struct SearchTab *tab, 
+GNUNET_GTK_add_search_result (struct SearchTab *tab,
+			      uint32_t anonymity,
                               GtkTreeRowReference *parent_rr,
                               const struct GNUNET_FS_Uri *uri,
                               const struct GNUNET_CONTAINER_MetaData *meta,
@@ -273,10 +309,23 @@ GNUNET_GTK_fs_event_handler (void *cls,
 
 /**
  * Close the 'uri_tab'.
- */ 
+ */
 void
 GNUNET_FS_GTK_close_uri_tab_ ();
 
 
+/**
+ * Update the connection indicator widget.
+ *
+ * @param main_ctx context
+ * @param connected TRUE if connected to arm, FALSE otherwise
+ * @param tooltip new tooltip text
+ */
+void
+GNUNET_FS_GTK_update_connection_indicator (
+    struct GNUNET_GTK_MainWindowContext *main_ctx,
+    gboolean connected, const gchar *tooltip);
+
+
 #endif
 /* end of gnunet-fs-gtk-event_handler.h */
