@@ -4,7 +4,7 @@
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 2, or (at your
+     by the Free Software Foundation; either version 3, or (at your
      option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
@@ -41,6 +41,11 @@ struct AddChildContext
    */
   struct SearchResult *sr;
 
+  /**
+   * Anonymity level to use for probes in the directory.
+   */
+  uint32_t anonymity;
+
 };
 
 
@@ -51,14 +56,14 @@ struct AddChildContext
  * @param cls closure, our 'struct AddChildContext*'
  * @param filename name of the file in the directory
  * @param uri URI of the file
- * @param metadata metadata for the file; metadata for
+ * @param meta metadata for the file; metadata for
  *        the directory if everything else is NULL/zero
  * @param length length of the available data for the file
  *           (of type size_t since data must certainly fit
  *            into memory; if files are larger than size_t
  *            permits, then they will certainly not be
  *            embedded with the directory itself).
- * @param data data available for the file (length bytes)
+ * @param data data available for the file (@a length bytes)
  */
 static void
 add_child (void *cls, const char *filename, const struct GNUNET_FS_Uri *uri,
@@ -78,7 +83,8 @@ add_child (void *cls, const char *filename, const struct GNUNET_FS_Uri *uri,
                                        EXTRACTOR_METAFORMAT_UTF8, "text/plain",
                                        acc->filename,
                                        strlen (acc->filename) + 1);
-    acc->sr = GNUNET_GTK_add_to_uri_tab (dmeta, NULL);
+    acc->sr = GNUNET_GTK_add_to_uri_tab (acc->anonymity,
+					 dmeta, NULL);
     GNUNET_CONTAINER_meta_data_destroy (dmeta);
     return;
   }
@@ -88,8 +94,9 @@ add_child (void *cls, const char *filename, const struct GNUNET_FS_Uri *uri,
     return;
   }
   GNUNET_assert (NULL !=
-                 GNUNET_GTK_add_search_result (acc->sr->tab, 
-					       acc->sr->rr, 
+                 GNUNET_GTK_add_search_result (acc->sr->tab,
+					       acc->anonymity,
+					       acc->sr->rr,
 					       uri,
                                                meta, NULL, 0));
 }
@@ -103,7 +110,7 @@ add_child (void *cls, const char *filename, const struct GNUNET_FS_Uri *uri,
  * @param user_data the builder of the dialog
  */
 void
-GNUNET_GTK_open_directory_dialog_response_cb (GtkDialog * dialog,
+GNUNET_GTK_open_directory_dialog_response_cb (GtkDialog *dialog,
                                               gint response_id,
                                               gpointer user_data)
 {
@@ -122,6 +129,7 @@ GNUNET_GTK_open_directory_dialog_response_cb (GtkDialog * dialog,
   g_object_unref (G_OBJECT (builder));
   acc.filename = filename;
   acc.sr = NULL;
+  acc.anonymity = 1; // FIXME, might want to add this to dialog to allow user to set it.
   GNUNET_FS_GTK_mmap_and_scan (filename, &add_child, &acc);
   GNUNET_free (filename);
 }
@@ -132,7 +140,7 @@ GNUNET_GTK_open_directory_dialog_response_cb (GtkDialog * dialog,
  * file and then display a new tab with its contents.
  *
  * @param dummy the menu entry
- * @param user_data the main dialog builder, unused
+ * @param data the main dialog builder, unused
  */
 void
 GNUNET_GTK_main_menu_file_open_gnunet_directory_activate_cb (GtkWidget * dummy,
