@@ -4,7 +4,7 @@
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 2, or (at your
+     by the Free Software Foundation; either version 3, or (at your
      option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
@@ -24,7 +24,9 @@
  * @author Christian Grothoff
  */
 #include "gnunet_gtk.h"
-
+#if HAVE_GTK_GTKX_H
+#include <gtk/gtkx.h>
+#endif
 
 /**
  * Initial size of our poll array cache.
@@ -100,6 +102,15 @@ struct GNUNET_GTK_MainLoop
    */
   GNUNET_SCHEDULER_TaskIdentifier dummy_task;
 
+  /**
+   * Remaining command-line arguments.
+   */
+  char *const*argv;
+
+  /**
+   * Number of remaining arguments.
+   */
+  int argc;
 
 #if WINDOWS
   /**
@@ -159,6 +170,45 @@ struct GNUNET_GTK_MainLoop
 };
 
 
+/**
+ * If desired, enable plugging of this main window in the meta window.
+ *
+ * @param env_name environment variable to check
+ * @param main_window main window to plug
+ * @return new main window to initialize
+ */
+GtkWidget *
+GNUNET_GTK_plug_me (const char *env_name,
+                    GtkWidget *main_window)
+{
+#if HAVE_GTK_GTKX_H
+  const char *plugid;
+  GtkWidget *plug;
+  GtkWidget *child;
+  unsigned long long id;
+
+  if (NULL == (plugid = getenv (env_name)))
+    return main_window; /* no plugging */
+  if (1 != sscanf (plugid,
+                   "%llu",
+                   &id))
+  {
+    fprintf (stderr,
+             _("Invalid plug name `%s'\n"),
+             plugid);
+    return main_window;
+  }
+  plug = gtk_plug_new ((Window) id);
+  child = gtk_bin_get_child (GTK_BIN (main_window));
+  gtk_widget_reparent (child,
+                       plug);
+  gtk_widget_destroy (main_window);
+  return plug;
+#else
+  return main_window;
+#endif
+}
+
 
 /**
  * Get the configuration.
@@ -203,8 +253,10 @@ GNUNET_GTK_main_loop_get_builder (struct GNUNET_GTK_MainLoop *ml)
   return ml->builder;
 }
 
+
 int
-GNUNET_GTK_main_loop_build_window (struct GNUNET_GTK_MainLoop *ml, gpointer data)
+GNUNET_GTK_main_loop_build_window (struct GNUNET_GTK_MainLoop *ml,
+				   gpointer data)
 {
   ml->builder = GNUNET_GTK_get_new_builder (ml->main_window_file, data);
   if (ml->builder == NULL)
@@ -244,6 +296,23 @@ GNUNET_GTK_main_loop_get_object (struct GNUNET_GTK_MainLoop * ml,
 }
 
 
+/**
+ * Get remaining command line arguments.
+ *
+ * @param ml handle to the main loop
+ * @param argc set to argument count
+ * @param argv set to argument vector
+ */
+void
+GNUNET_GTK_main_loop_get_args (struct GNUNET_GTK_MainLoop * ml,
+			       int *argc,
+			       char *const**argv)
+{
+  *argc = ml->argc;
+  *argv = ml->argv;
+}
+
+
 /**
  * Task to run Gtk events (within a GNUnet scheduler task).
  *
@@ -590,14 +659,14 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
       resize_cached_poll_array (ml, fd_counter + need_gfds);
     }
   }
-  if (timeout.rel_value != GNUNET_TIME_UNIT_FOREVER_REL.rel_value)
+  if (timeout.rel_value_us != GNUNET_TIME_UNIT_FOREVER_REL.rel_value_us)
   {
     if (delay >= 0)
       delay =
-          GNUNET_MIN (timeout.rel_value /
-                      GNUNET_TIME_UNIT_MILLISECONDS.rel_value, delay);
+          GNUNET_MIN (timeout.rel_value_us /
+                      GNUNET_TIME_UNIT_MILLISECONDS.rel_value_us, delay);
     else
-      delay = timeout.rel_value / GNUNET_TIME_UNIT_MILLISECONDS.rel_value;
+      delay = timeout.rel_value_us / GNUNET_TIME_UNIT_MILLISECONDS.rel_value_us;
   }
 
 #if WINDOWS
@@ -816,7 +885,7 @@ gnunet_gtk_select (void *cls, struct GNUNET_NETWORK_FDSet *rfds,
  * is initialized.  Initializes up GTK and Glade and starts the
  * combined event loop.
  *
- * @param cls the 'struct GNUNET_GTK_MainLoop'
+ * @param cls the `struct GNUNET_GTK_MainLoop`
  * @param args leftover command line arguments (go to gtk)
  * @param cfgfile name of the configuration file
  * @param cfg handle to the configuration
@@ -855,7 +924,7 @@ run_main_loop (void *cls, char *const *args, const char *cfgfile,
   }
   if (0 != strcmp (ipath, ipath2))
   {
-    GNUNET_asprintf (&baseconfig, "%s%s", ipath, "config.d");
+    GNUNET_asprintf (&baseconfig, "%s%s", ipath2, "config.d");
     if (GNUNET_YES ==
 	GNUNET_DISK_directory_test (baseconfig, GNUNET_YES))
       (void) GNUNET_CONFIGURATION_load_from (gcfg, baseconfig);
@@ -864,13 +933,17 @@ run_main_loop (void *cls, char *const *args, const char *cfgfile,
   GNUNET_free (ipath2);
   GNUNET_free (ipath);
 
-  (void) GNUNET_CONFIGURATION_load (gcfg, cfgfile);
+  if (GNUNET_YES ==
+      GNUNET_DISK_file_test (cfgfile))
+    (void) GNUNET_CONFIGURATION_parse (gcfg, cfgfile);
   ml->rs = GNUNET_NETWORK_fdset_create ();
   ml->ws = GNUNET_NETWORK_fdset_create ();
   ml->gml = g_main_loop_new (NULL, TRUE);
   ml->gmc = g_main_loop_get_context (ml->gml);
   ml->cfg = gcfg;
   ml->cfgfile = GNUNET_strdup (cfgfile);
+  ml->argc = argc;
+  ml->argv = args;
 #if WINDOWS
   ml->hEventRead = CreateEvent (NULL, TRUE, FALSE, NULL);
   ml->hEventReadReady = CreateEvent (NULL, TRUE, TRUE, NULL);
@@ -884,10 +957,6 @@ run_main_loop (void *cls, char *const *args, const char *cfgfile,
   ml->read_array_length = 0;
 #endif
 
-  /* run main task of the application */
-  GNUNET_SCHEDULER_add_continuation (ml->main_task, ml,
-                                     GNUNET_SCHEDULER_REASON_STARTUP);
-
   /* start the Gtk event loop */
   GNUNET_assert (TRUE == g_main_context_acquire (ml->gmc));
   GNUNET_SCHEDULER_set_select (&gnunet_gtk_select, ml);
@@ -897,6 +966,9 @@ run_main_loop (void *cls, char *const *args, const char *cfgfile,
       GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
                                     &keepalive_task, ml);
 
+  /* run main task of the application */
+  GNUNET_SCHEDULER_add_continuation (ml->main_task, ml,
+                                     GNUNET_SCHEDULER_REASON_STARTUP);
 }
 
 
@@ -909,8 +981,8 @@ run_main_loop (void *cls, char *const *args, const char *cfgfile,
  * @param argv command line options
  * @param options allowed command line options
  * @param main_window_file glade file for the main window
- * @param main_task first task to run, closure will be set to the 'struct GNUNET_GTK_MainLoop'
- * @return GNUNET_OK on success, GNUNET_SYSERR on error (i.e. bad command-line options, etc)
+ * @param main_task first task to run, closure will be set to the `struct GNUNET_GTK_MainLoop`
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on error (i.e. bad command-line options, etc)
  */
 int
 GNUNET_GTK_main_loop_start (const char *binary_name, const char *binary_help,
