@@ -1,10 +1,10 @@
 /*
      This file is part of GNUnet.
-     (C) 2010, 2011, 2012 Christian Grothoff (and other contributing authors)
+     (C) 2010, 2011, 2012, 2013 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
-     by the Free Software Foundation; either version 2, or (at your
+     by the Free Software Foundation; either version 3, or (at your
      option) any later version.
 
      GNUnet is distributed in the hope that it will be useful, but
@@ -26,7 +26,6 @@
 #include "gnunet-setup.h"
 #include "gnunet-setup-transport.h"
 #include "gnunet-setup-options.h"
-#include "gnunet-setup-gns.h"
 #include <gnunet/gnunet_constants.h>
 #include <regex.h>
 #if ENABLE_NLS
@@ -42,7 +41,7 @@ static struct GNUNET_GTK_MainLoop *ml;
 /**
  * Name of the configuration file.
  */
-static const char *cfgName;
+static const char *option_cfg_name;
 
 /**
  * Our configuration.
@@ -64,6 +63,11 @@ static struct GNUNET_OS_Process *resolver;
  */
 static struct GNUNET_OS_Process *namestore;
 
+/**
+ * Identity process handle.
+ */
+static struct GNUNET_OS_Process *identity;
+
 /**
  * Run autoconfig-only and then exit immediately.
  */
@@ -74,13 +78,6 @@ static int do_autoconfig;
  * Flag to enable privilege escalation.
  */
 static int do_gksu;
-
-/**
- * Show all configuration options, even if we are run
- * as a normal user and a 'gnunet' user exists and thus
- * most options should not apply to the normal user.
- */ 
-static int force_full_setup;
 #endif
 
 
@@ -125,23 +122,23 @@ help_click_callback (GtkWidget * widget, GdkEventButton * event,
 
 /**
  * The main visible page in our main notebook changed.  If the
- * GADS page is visible, hide the help text, otherwise show it.
+ * GNS page is visible, hide the help text, otherwise show it.
  */
 void
 GNUNET_setup_notebook_switch_page_cb (GtkNotebook *notebook,
 				      GtkWidget   *page,
 				      guint        page_num,
-				      gpointer     user_data) 
+				      gpointer     user_data)
 {
   GtkWidget *help;
-  GtkWidget *gads;
+  GtkWidget *gnu;
 
   help = GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_help_text"));
-  gads = GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_gads_vbox"));
-  if (gads == page)
+  gnu = GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_gns_vbox"));
+  if (gnu == page)
     gtk_widget_hide (help);
   else
-    gtk_widget_show (help);  
+    gtk_widget_show (help);
 }
 
 
@@ -255,7 +252,7 @@ load_options ()
   GObject *widget;
   char *value;
 
-#ifndef LINUX 
+#ifndef LINUX
   gtk_widget_hide (GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_gns_hijack_checkbutton")));
 #endif
   i = 0;
@@ -333,9 +330,9 @@ load_options ()
 /**
  * Write final configuration to disk.
  *
- * @return GNUNET_OK on success
+ * @return #GNUNET_OK on success
  */
-static int 
+static int
 write_configuration ()
 {
   struct GNUNET_CONFIGURATION_Handle *cfgDefault;
@@ -343,14 +340,15 @@ write_configuration ()
 
   cfgDefault = GNUNET_CONFIGURATION_create ();
   (void) GNUNET_CONFIGURATION_load (cfgDefault, NULL);  /* load defaults only */
-  ret = GNUNET_CONFIGURATION_write_diffs (cfgDefault, cfg, cfgName);
+  ret = GNUNET_CONFIGURATION_write_diffs (cfgDefault, cfg,
+                                          option_cfg_name);
   GNUNET_CONFIGURATION_destroy (cfgDefault);
   return ret;
 }
 
 
 /**
- * Actual main method that sets up the configuration window.
+ * Method run on shutdown.
  *
  * @param cls the main loop handle
  * @param tc scheduler context
@@ -363,7 +361,6 @@ cleanup_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     GNUNET_break (0);
     return;
   }
-  GNUNET_SETUP_gns_done ();
   GNUNET_GTK_main_loop_quit (ml);
   ml = NULL;
   if (GNUNET_OK != write_configuration ())
@@ -381,13 +378,19 @@ cleanup_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     GNUNET_OS_process_destroy (namestore);
     namestore = NULL;
   }
+  if (NULL != identity)
+  {
+    GNUNET_break (0 == GNUNET_OS_process_kill (identity, SIGTERM));
+    GNUNET_OS_process_destroy (identity);
+    identity = NULL;
+  }
 }
 
 
 /**
  * Write configuration to dis, (re)start the namestore process and
  * reload the namestore models.
- * 
+ *
  *
  * @param cls closure (unused)
  * @param tc scheduler context (unused)
@@ -398,7 +401,6 @@ GNUNET_SETUP_restart_namestore (void *cls,
 {
   if (GNUNET_OK != write_configuration ())
     return; /* no point in re-starting namestore ... */
-  GNUNET_SETUP_gns_done ();
   if (NULL != namestore)
   {
     GNUNET_break (0 == GNUNET_OS_process_kill (namestore, SIGTERM));
@@ -411,7 +413,6 @@ GNUNET_SETUP_restart_namestore (void *cls,
                              NULL, NULL,
                              "gnunet-service-namestore",
 			     "gnunet-service-namestore", NULL);
-  GNUNET_SETUP_gns_init ();
 }
 
 
@@ -429,16 +430,15 @@ GNUNET_SETUP_quit_cb (GObject * object, gpointer user_data)
  * If the test failed, start the resolver process.
  *
  * @param cls closure, NULL
- * @param tc task context, reason is TIMEOUT if we need to
- *                start the process
+ * @param result #GNUNET_YES if the service is running
  */
 static void
 start_resolver (void *cls,
-		const struct GNUNET_SCHEDULER_TaskContext *tc)
+		int result)
 {
   char *binary;
 
-  if (0 == (tc->reason & GNUNET_SCHEDULER_REASON_TIMEOUT))
+  if (GNUNET_YES == result)
     return;
   binary = GNUNET_OS_get_libexec_binary_path ("gnunet-service-resolver");
   resolver =
@@ -451,21 +451,42 @@ start_resolver (void *cls,
 
 
 /**
- * If the test failed, start the namestore process.
+ * If the test failed, start the identity process.
  *
  * @param cls closure, NULL
- * @param tc task context, reason is TIMEOUT if we need to
- *                start the process
+ * @param result #GNUNET_YES if the service is running
  */
 static void
-start_namestore (void *cls,
-		 const struct GNUNET_SCHEDULER_TaskContext *tc)
+start_identity (void *cls,
+		int result)
 {
   char *binary;
 
-  if (0 == (tc->reason & GNUNET_SCHEDULER_REASON_TIMEOUT))
+  if (GNUNET_YES == result)
     return;
-  if (0 != (tc->reason & GNUNET_SCHEDULER_REASON_SHUTDOWN))
+  binary = GNUNET_OS_get_libexec_binary_path ("gnunet-service-identity");
+  identity =
+    GNUNET_OS_start_process (GNUNET_YES, GNUNET_OS_INHERIT_STD_ALL,
+                             NULL, NULL,
+                             binary,
+			     "gnunet-service-identity", NULL);
+  GNUNET_free (binary);
+}
+
+
+/**
+ * If the test failed, start the namestore process.
+ *
+ * @param cls closure, NULL
+ * @param result #GNUNET_YES if the service is running
+ */
+static void
+start_namestore (void *cls,
+		 int result)
+{
+  char *binary;
+
+  if (GNUNET_YES == result)
     return;
   binary = GNUNET_OS_get_libexec_binary_path ("gnunet-service-namestore");
   namestore =
@@ -507,9 +528,36 @@ try_gksu (const char *username)
 #endif
 
 
+/**
+ * User clicked on the button to edit the list of friends.
+ * Launch gnunet-peerinfo-gtk.
+ */
+void
+GNUNET_setup_launch_edit_friends_button_clicked_cb (GtkButton *button,
+                                                    gpointer *user_data)
+{
+  struct GNUNET_OS_Process *proc;
+
+  proc = GNUNET_OS_start_process (GNUNET_NO,
+				  GNUNET_OS_INHERIT_STD_ALL,
+				  NULL, NULL,
+				  "gnunet-peerinfo-gtk",
+				  "-c", option_cfg_name,
+                                  NULL);
+  if (NULL == proc)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                _("Failed to launch gnunet-peerinfo-gtk\n"));
+    return;
+  }
+  /* simply yield control, let it run */
+  GNUNET_free (proc);
+}
+
+
 /**
  * Called once the autoconfiguration is done.  Triggers shutdown.
- * 
+ *
  * @param cls unused
  */
 static void
@@ -526,7 +574,8 @@ autoconfig_finished_cb (void *cls)
  * @param tc scheduler context
  */
 static void
-run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
+run (void *cls,
+     const struct GNUNET_SCHEDULER_TaskContext *tc)
 {
   GtkWidget *main_window;
 #ifndef MINGW
@@ -545,55 +594,22 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
     GNUNET_GTK_main_loop_quit (cls);
     return;
   }
-#endif  
+#endif
 
   ml = cls;
   if (GNUNET_OK != GNUNET_GTK_main_loop_build_window (ml, NULL))
     return;
 
-  cfgName = GNUNET_GTK_main_loop_get_configuration_file (ml);
+  option_cfg_name = GNUNET_GTK_main_loop_get_configuration_file (ml);
   cfg = GNUNET_CONFIGURATION_create ();
-  
-#ifndef MINGW  
-  if ( (0 != my_uid) &&
-       (NULL != gnunet_pw) &&
-       (my_uid != gnunet_pw->pw_uid) &&
-       (0 == force_full_setup) )
-  {
-    /* load system defaults, system/'gnunet' user's configuration and THEN
-       our configuration file */
-    if (GNUNET_YES ==
-	GNUNET_DISK_file_test ("/etc/gnunet.conf"))
-      (void) GNUNET_CONFIGURATION_load (cfg, "/etc/gnunet.conf");
-    else
-    {
-#if HAVE_GETPWNAM
-      struct passwd *pw;
-
-      pw = getpwnam ("gnunet");
-      if (NULL != pw)
-      {
-	char *cfgname;
-
-	GNUNET_asprintf (&cfgname,
-			 "%s%s%s",
-			 pw->pw_dir,
-			 DIR_SEPARATOR_STR,
-			 ".gnunet/gnunet.conf");
-	(void) GNUNET_CONFIGURATION_load (cfg, cfgname);
-	GNUNET_free (cfgname);
-      }
-#endif
-    }
-    (void) GNUNET_CONFIGURATION_parse (cfg, cfgName);
-  }
+  if (GNUNET_YES ==
+      GNUNET_DISK_file_test (option_cfg_name))
+    (void) GNUNET_CONFIGURATION_load (cfg, option_cfg_name);
   else
-#endif
-  {
-    /* only load system defaults and our configuration file */
-    (void) GNUNET_CONFIGURATION_load (cfg, cfgName);
-  }
+    (void) GNUNET_CONFIGURATION_load (cfg, NULL);
   main_window = GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_dialog"));
+  main_window = GNUNET_GTK_plug_me ("GNUNET_FS_GTK_PLUG",
+                                    main_window);
   GNUNET_CLIENT_service_test ("resolver",
 			      cfg,
 			      GNUNET_CONSTANTS_SERVICE_TIMEOUT,
@@ -602,25 +618,11 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
 			      cfg,
 			      GNUNET_CONSTANTS_SERVICE_TIMEOUT,
 			      &start_namestore, NULL);
+  GNUNET_CLIENT_service_test ("identity",
+			      cfg,
+			      GNUNET_CONSTANTS_SERVICE_TIMEOUT,
+			      &start_identity, NULL);
   load_options ();
-  GNUNET_SETUP_gns_init ();
-#ifndef MINGW
-  if ( (0 != my_uid) &&
-       (NULL != gnunet_pw) &&
-       (my_uid != gnunet_pw->pw_uid) &&
-       (0 == force_full_setup) )
-  {
-    /* hide all options that are not for individual users; this
-       run is only for per-user options, not for per-peer options */
-    gtk_widget_hide (GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_general_vbox")));
-    gtk_widget_hide (GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_transport_main_frame")));
-    gtk_widget_hide (GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_network_tab_vbox")));
-    gtk_widget_hide (GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_fs_main_vbox")));
-    gtk_widget_hide (GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_vpn_vbox")));
-    gtk_widget_hide (GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_exit_vbox")));
-    gtk_widget_hide (GTK_WIDGET (GNUNET_SETUP_get_object ("GNUNET_setup_namestore_vbox")));
-  }
-#endif
   GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
 				&cleanup_task, NULL);
   if (do_autoconfig)
@@ -653,9 +655,6 @@ main (int argc, char *const *argv)
     { 'e', "elevate-priviledges", NULL,
       gettext_noop ("run as user 'gnunet', if necessary by executing gksu to elevate rights"),
       0, &GNUNET_GETOPT_set_one, &do_gksu },
-    { 'f', "force-full-setup", NULL,
-      gettext_noop ("force showing the full set of options, even if gnunet-setup is run as a normal user and as a user 'gnunet' exists on the system, most options should not apply to the normal user as GNUnet should be run by the 'gnunet' user"),
-      0, &GNUNET_GETOPT_set_one, &force_full_setup },
 #endif
     GNUNET_GETOPT_OPTION_END
   };
@@ -663,7 +662,7 @@ main (int argc, char *const *argv)
 
   if (GNUNET_OK ==
       GNUNET_GTK_main_loop_start ("gnunet-setup", "gnunet-setup", argc, argv,
-                                  options, "gnunet_setup_gtk_main_window.glade",
+                                  options, "gnunet_setup_main_window.glade",
                                   &run))
     ret = gret;
   else
